package be.jeffcheasey88.peeratcode.parser.java.operations;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import be.jeffcheasey88.peeratcode.parser.java.ArrayBuffer;
import be.jeffcheasey88.peeratcode.parser.java.CleanerPool;
import be.jeffcheasey88.peeratcode.parser.java.JavaElement;

public class TryCatchOperation extends OperationContainer{
	
	private static Pattern PATTERN = Pattern.compile("^(\\^GENERIC_FUNCTION\\d+)");
	private static Pattern PATTERN_SPACE = Pattern.compile("^(\\s+)");
	
	public TryCatchOperation(){}
	
	@Override
	public int parse(String content, CleanerPool global, CleanerPool local) throws Exception{
		Matcher matcher = PATTERN.matcher(content);
		matcher.lookingAt();
		
		content = matcher.group(1);
		
		int bodysize = content.length();
		content = local.unzipOne(content, (s,p) -> s);
		content = content.substring(1, content.length()-1);
		content = local.clean(content);
		super.parse(content, global, local);
		
		return bodysize;
	}

	public static class TryOperation extends TryCatchOperation{
		
		private static Pattern PATTERN = Pattern.compile("^(\\s*try\\s*(\\^GENERIC_PARENTHESIS\\d+)?)");
	
		private String resource;
		
		public TryOperation(){}
		
		@Override
		public int parse(String content, CleanerPool global, CleanerPool local) throws Exception{
			Matcher matcher = TryOperation.PATTERN.matcher(content);
			matcher.lookingAt();
			
			if(matcher.group(2) != null) this.resource = local.unzipOne(matcher.group(2), (s,p) -> s);
			
			int index = matcher.group(1).length();
			content = content.substring(index);
			
			matcher = PATTERN_SPACE.matcher(content);
			if(matcher.lookingAt()){
				index+=matcher.group(1).length();
				content = content.substring(matcher.group(1).length());
			}
			
			return index+super.parse(content, global, local);
		}
		
		@Override
		public void build(ArrayBuffer<String> buffer, int tab) throws Exception{
			super.build(buffer, tab);
			buffer.append((s) -> s+="try"+(resource == null ? "":resource)+"{");
			buffer.add("");
			
			for(JavaElement child : getChilds()) child.build(buffer, tab+1);
			
			super.build(buffer, tab);
			buffer.append((s) -> s+="}");
			buffer.add("");
		}
	}
	
	public static class CatchOperation extends TryCatchOperation{
		
		private static Pattern PATTERN = Pattern.compile("^(\\s*catch\\s*(\\^GENERIC_PARENTHESIS\\d+))");
	
		private String exceptions;
		
		public CatchOperation(){}
		
		@Override
		public int parse(String content, CleanerPool global, CleanerPool local) throws Exception{
			Matcher matcher = CatchOperation.PATTERN.matcher(content);
			matcher.lookingAt();
			
			this.exceptions = local.unzipOne(matcher.group(2), (s,p) -> s);
			
			int index = matcher.group(1).length();
			content = content.substring(index);
			
			matcher = PATTERN_SPACE.matcher(content);
			if(matcher.lookingAt()){
				index+=matcher.group(1).length();
				content = content.substring(matcher.group(1).length());
			}
			
			return index+super.parse(content, global, local);
		}
		
		@Override
		public void build(ArrayBuffer<String> buffer, int tab) throws Exception{
			super.build(buffer, tab);
			buffer.append((s) -> s+="catch"+exceptions+"{");
			buffer.add("");
			
			for(JavaElement child : getChilds()) child.build(buffer, tab+1);
			
			super.build(buffer, tab);
			buffer.append((s) -> s+="}");
			buffer.add("");
		}
	}
	
	public static class FinallyOperation extends TryCatchOperation{
		
		private static Pattern PATTERN = Pattern.compile("^(\\s*finally\\s*)");
	
		public FinallyOperation(){}
		
		@Override
		public int parse(String content, CleanerPool global, CleanerPool local) throws Exception{
			Matcher matcher = FinallyOperation.PATTERN.matcher(content);
			matcher.lookingAt();
			
			int index = matcher.group(1).length();
			content = content.substring(index);
			
			matcher = PATTERN_SPACE.matcher(content);
			if(matcher.lookingAt()){
				index+=matcher.group(1).length();
				content = content.substring(matcher.group(1).length());
			}
			
			return index+super.parse(content, global, local);
		}
		
		@Override
		public void build(ArrayBuffer<String> buffer, int tab) throws Exception{
			super.build(buffer, tab);
			buffer.append((s) -> s+="finally{");
			buffer.add("");
			
			for(JavaElement child : getChilds()) child.build(buffer, tab+1);
			
			super.build(buffer, tab);
			buffer.append((s) -> s+="}");
			buffer.add("");
		}
	}
}
